@@ -99,6 +99,10 @@ class EOCRecorder:
 
             tbl.add_row((absc_str, err_str, eoc_str))
 
+        if len(self.history) > 1:
+            order = self.estimate_order_of_convergence()[0, 1]
+            tbl.add_row(("Overall", "", eoc_format % order))
+
         return tbl
 
     def pretty_print(self, *,
@@ -117,22 +121,16 @@ class EOCRecorder:
                 gliding_mean=gliding_mean)
 
         if table_type == "markdown":
-            tbl_str = tbl.github_markdown()
+            return tbl.github_markdown()
         elif table_type == "latex":
-            tbl_str = tbl.latex()
+            return tbl.latex()
         elif table_type == "ascii":
-            tbl_str = str(tbl)
+            return str(tbl)
         elif table_type == "csv":
-            tbl_str = tbl.csv()
+            return tbl.csv()
         else:
             raise ValueError(f"unknown table type: {table_type}")
 
-        if len(self.history) > 1:
-            return "{}\n\nOverall EOC: {}".format(tbl_str,
-                    self.estimate_order_of_convergence()[0, 1])
-        else:
-            return tbl_str
-
     def __str__(self):
         return self.pretty_print()
 
@@ -147,6 +145,49 @@ class EOCRecorder:
         for absc, _err in self.history:
             outfile.write(f"{absc:f} {const * absc**(-order):f}\n")
 
+
+def stringify_eocs(*eocs: EOCRecorder,
+        names: Optional[Tuple[str, ...]] = None,
+        abscissa_label: str = "h",
+        error_label: str = "Error",
+        gliding_mean: int = 2,
+        abscissa_format: str = "%s",
+        error_format: str = "%s",
+        eoc_format: str = "%s",
+        table_type: str = "markdown") -> str:
+    """
+    :arg names: a :class:`tuple` of names to use for the *error_label* of each
+        *eoc*.
+    """
+    if names is not None and len(names) < len(eocs):
+        raise ValueError(
+                f"insufficient names: got {len(names)} names for "
+                f"{len(eocs)} EOCRecorder instances")
+
+    if names is None:
+        names = tuple([f"{error_label} {i}" for i in range(len(eocs))])
+
+    from pytools import merge_tables
+    tbl = merge_tables(*[eoc._to_table(
+        abscissa_label=abscissa_label, error_label=name,
+        abscissa_format=abscissa_format,
+        error_format=error_format,
+        eoc_format=eoc_format,
+        gliding_mean=gliding_mean)
+        for name, eoc in zip(names, eocs)
+        ], skip_columns=(0,))
+
+    if table_type == "markdown":
+        return tbl.github_markdown()
+    elif table_type == "latex":
+        return tbl.latex()
+    elif table_type == "ascii":
+        return str(tbl)
+    elif table_type == "csv":
+        return tbl.csv()
+    else:
+        raise ValueError(f"unknown table type: {table_type}")
+
 # }}}
 
 
