@@ -1,5 +1,3 @@
-from __future__ import division, with_statement
-
 __copyright__ = "Copyright (C) 2009-2013 Andreas Kloeckner"
 
 __license__ = """
@@ -25,22 +23,18 @@ THE SOFTWARE.
 import marshal
 from types import FunctionType, ModuleType
 
-import six
-
 from pytools.codegen import CodeGenerator as CodeGeneratorBase
 from pytools.codegen import Indentation, remove_common_indentation  # noqa
 
 
-try:
-    from importlib.util import MAGIC_NUMBER as BYTECODE_VERSION
-except ImportError:
-    # Pre-3.4
-    import imp
-    BYTECODE_VERSION = imp.get_magic()
+from importlib.util import MAGIC_NUMBER as BYTECODE_VERSION
 
 
 class PythonCodeGenerator(CodeGeneratorBase):
-    def get_module(self, name="<generated code>"):
+    def get_module(self, name=None):
+        if name is None:
+            name = "<generated code>"
+
         result_dict = {}
         source_text = self.get()
         exec(compile(
@@ -49,8 +43,8 @@ class PythonCodeGenerator(CodeGeneratorBase):
         result_dict["_MODULE_SOURCE_CODE"] = source_text
         return result_dict
 
-    def get_picklable_module(self):
-        return PicklableModule(self.get_module())
+    def get_picklable_module(self, name=None):
+        return PicklableModule(self.get_module(name=name))
 
 
 class PythonFunctionGenerator(PythonCodeGenerator):
@@ -58,14 +52,18 @@ class PythonFunctionGenerator(PythonCodeGenerator):
         PythonCodeGenerator.__init__(self)
         self.name = name
 
-        self("def %s(%s):" % (name, ", ".join(args)))
+        self("def {}({}):".format(name, ", ".join(args)))
         self.indent()
 
+    @property
+    def _gen_filename(self):
+        return f"<generated code for '{self.name}'>"
+
     def get_function(self):
-        return self.get_module()[self.name]
+        return self.get_module(name=self._gen_filename)[self.name]
 
     def get_picklable_function(self):
-        module = self.get_picklable_module()
+        module = self.get_picklable_module(name=self._gen_filename)
         return PicklableFunction(module, self.name)
 
 
@@ -80,7 +78,7 @@ def _get_empty_module_dict():
 _empty_module_dict = _get_empty_module_dict()
 
 
-class PicklableModule(object):
+class PicklableModule:
     def __init__(self, mod_globals):
         self.mod_globals = mod_globals
 
@@ -89,7 +87,7 @@ class PicklableModule(object):
         functions = {}
         modules = {}
 
-        for k, v in six.iteritems(self.mod_globals):
+        for k, v in self.mod_globals.items():
             if isinstance(v, FunctionType):
                 functions[k] = (
                         v.__name__,
@@ -112,18 +110,18 @@ class PicklableModule(object):
             raise ValueError("unknown version of PicklableModule")
 
         if magic != BYTECODE_VERSION:
-            raise ValueError("cannot unpickle function binary: "
-                    "incorrect magic value (got: %r, expected: %r)"
-                    % (magic, BYTECODE_VERSION))
+            raise ValueError(
+                    "cannot unpickle function binary: incorrect magic value "
+                    f"(got: {magic!r}, expected: {BYTECODE_VERSION!r})")
 
         mod_globals = _empty_module_dict.copy()
         mod_globals.update(nondefault_globals)
 
-        from pytools.importlib_backport import import_module
-        for k, mod_name in six.iteritems(modules):
+        from importlib import import_module
+        for k, mod_name in modules.items():
             mod_globals[k] = import_module(mod_name)
 
-        for k, (name, code_bytes, argdefs) in six.iteritems(functions):
+        for k, (name, code_bytes, argdefs) in functions.items():
             f = FunctionType(
                     marshal.loads(code_bytes), mod_globals, name=name,
                     argdefs=argdefs)
@@ -136,7 +134,7 @@ class PicklableModule(object):
 
 # {{{ picklable function
 
-class PicklableFunction(object):
+class PicklableFunction:
     """Convience class wrapping a function in a :class:`PicklableModule`.
     """
 
